//This file is a part of xboa
//
//xboa is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//xboa is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with xboa in the doc folder.  If not, see 
//<http://www.gnu.org/licenses/>.
//

/*
Testcore.c

Excercise Hitcore functions in order to find stinking memory leak (in
combination with valgrind).
*/

#include <Python.h>
#include "Hitcore.c"

void test(int test_condition, char* function_tested, char* test_name) {
  printf("%s   %s", function_tested, test_name);
  if (!test_condition) {
    puts("         fail\n");
    exit(1);
  }
  printf("         pass\n");
}

//typedef struct {void* data; size_t data_length; size_t allocd_length; size_t object_size; } vector;

//static vector* vector_alloc   (size_t object_size, int n_elements);
void test_vector_alloc() {
  char* function_tested = "vector_alloc";
  vector* vec_zero = vector_alloc(0, 1);
  test(vec_zero == NULL, function_tested, "0 size object");

  vector* vec_no_data = vector_alloc(2, 0);
  test(vec_no_data != NULL, function_tested,             "0 length vector alloc");
  test(vec_no_data->data != NULL, function_tested,       "0 length vector data");
  test(vec_no_data->data_length == 0, function_tested,   "0 length vector data_length");
  test(vec_no_data->allocd_length == 1, function_tested, "0 length vector allocd_length");
  test(vec_no_data->object_size == 2, function_tested,   "0 length vector object_size");
  vector_free(vec_no_data);

  vector* vec_normal = vector_alloc(2, 3);
  test(vec_normal != NULL,             function_tested, "Standard vector alloc");
  test(vec_normal->data != NULL,       function_tested, "Standard vector data");
  test(vec_normal->data_length == 6,   function_tested, "Standard vector data_length");
  test(vec_normal->allocd_length == 7, function_tested, "Standard vector allocd_length");
  test(vec_normal->object_size == 2,   function_tested, "Standard vector object_size");
  vector_free(vec_normal);
}

void test_vector_realloc() {
  char* function_tested = "vector_realloc";
  vector* vec_normal = vector_alloc(2, 3);
  vector* vec_no_data = vector_alloc(2, 0);

  vector_realloc(vec_normal, 1); //try making it shorter
  test(vec_normal != NULL,             function_tested, "Normal vector");
  test(vec_normal->data != NULL,       function_tested, "Normal vector data");
  test(vec_normal->data_length == 2,   function_tested, "Normal vector data_length");
  test(vec_normal->allocd_length == 3, function_tested, "Normal vector allocd_length");
  test(vec_normal->object_size == 2,   function_tested, "Normal vector object_size");
  
  vector_realloc(vec_normal, 5); //try making it longer
  test(vec_normal != NULL,              function_tested, "Normal vector");
  test(vec_normal->data != NULL,        function_tested, "Normal vector data");
  test(vec_normal->data_length == 10,   function_tested, "Normal vector data_length");
  test(vec_normal->allocd_length == 11, function_tested, "Normal vector allocd_length");
  test(vec_normal->object_size == 2,    function_tested, "Normal vector object_size");
  vector_free(vec_normal);

  vector_realloc(vec_no_data, 5); //try making it longer
  test(vec_no_data != NULL,              function_tested, "no_data vector");
  test(vec_no_data->data != NULL,        function_tested, "no_data vector data");
  test(vec_no_data->data_length == 10,   function_tested, "no_data vector data_length");
  test(vec_no_data->allocd_length == 11, function_tested, "no_data vector allocd_length");
  test(vec_no_data->object_size == 2,    function_tested, "no_data vector object_size");
  vector_free(vec_no_data);
}

void test_vector_init() {
  char* function_tested = "vector_init";
  double* data = malloc(sizeof(double[3]));
  vector* vec_init = vector_init(data, sizeof(double)*2, sizeof(double)*3, sizeof(double));
  test(vec_init != NULL,              function_tested, "vec_init vector");
  test(vec_init->data != NULL,        function_tested, "vec_init vector data");
  test(vec_init->data_length == sizeof(double)*2,     function_tested, "vec_init vector data_length");
  test(vec_init->allocd_length == sizeof(double)*3, function_tested, "vec_init vector allocd_length");
  test(vec_init->object_size == sizeof(double),       function_tested, "vec_init vector object_size");
  vector_free(vec_init);

  double* no_data = NULL;
  vector* vec_no_length = vector_init(no_data, 0, 0, 0);
  test(vec_no_length == NULL,  function_tested, "vec_no_length vector");

  vector* vec_no_data = vector_init(no_data, 0, 0, sizeof(double));
  test(vec_no_data != NULL,                        function_tested, "vec_no_data vector");
  test(vec_no_data->data == NULL,                  function_tested, "vec_no_data vector data");
  test(vec_no_data->data_length == 0,              function_tested, "vec_no_data vector data_length");
  test(vec_no_data->allocd_length == 0,            function_tested, "vec_no_data vector allocd_length");
  test(vec_no_data->object_size == sizeof(double), function_tested, "vec_no_data vector object_size");
  vector_free(vec_no_data);
}

void test_vector_insert() {
  char* function_tested = "vector_insert";
  int ierr = 0;

  double* data_1 = malloc(sizeof(double[3]));
  vector* vector_1 = vector_init(data_1, sizeof(double)*3, sizeof(double)*3, sizeof(double));
  double insert_1[2] = {0.,1.};
  vector_insert(vector_1, insert_1, insert_1+sizeof(insert_1), 0, &ierr);
  double* p_1 = vector_el(vector_1, 0);
  test(p_1[0] == 0., function_tested, "vec_insert data 0");
  test(p_1[1] == 1., function_tested, "vec_insert data 1");
  vector_free(vector_1);

  // crashes - not sure why
  vector* vector_2 = vector_init(NULL, 0, 0, sizeof(double));
  double insert_2[2] = {0., 1.};
  double* p_2 = vector_el(vector_2, 0);
  vector_insert(vector_2, insert_2, insert_2+sizeof(insert_2), 0, &ierr);
  test(p_2[0] == 0., function_tested, "vec_insert data 0");
  test(p_2[1] == 1., function_tested, "vec_insert data 1");
  vector_free(vector_2);

  double* data_3 = malloc(sizeof(double[3]));
  vector* vector_3 = vector_init(data_3, sizeof(double)*3, sizeof(double)*3, sizeof(double));
  double insert_3[6] = {0., 1., 2., 3., 4., 5.};
  vector_insert(vector_3, insert_3, insert_3+sizeof(insert_3), sizeof(double)*3, &ierr);
  double* p_3 = vector_el(vector_3, 0);
  test(p_3[3] == 0., function_tested, "vec_insert data 3");
  test(p_3[4] == 1., function_tested, "vec_insert data 4");
  test(p_3[8] == 5., function_tested, "vec_insert data 8");
  vector_free(vector_3);
}

/*
vector vec_non-zero = vector_alloc(2, 2);
//realloc the vec
static vector* vector_realloc (vector* vec, int n_elements);
//build a vector from allocated data - data must be allocated elsewhere
static vector* vector_init    (void* data, size_t data_length, size_t allocd_length, size_t object_size);
//delete the vector
static void    vector_free (vector* vec);
//insert elements found between target_start and target_end into vec at the point vec->data+insert_point
//if insert_start == NULL, inserts at end (appends)
//fail if runs out of memory
static void  vector_insert    (vector* vec, void* start, void* end, size_t insert_point, int* ierr);
//return vector element at point int
static void* vector_el        (vector* vec, int element);
//return vector element at point int - with bound checking, returns NULL on out-of-bounds
static void* vector_el_bc     (vector* vec, int element);
//return number of elements
static size_t   vector_size      (vector* vec) 
*/

int main() {
  test_vector_alloc();
  test_vector_realloc();
  test_vector_init();
  test_vector_insert();
  return 0;
}


